package com.perforce.hws.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Common buffering and copying operations on InputStream and OutputStream
 */
public interface StreamUtils {

    default byte[] readAll(InputStream is) throws IOException {
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        copyStream(is, os);
        os.flush();
        return os.toByteArray();
    }

    default long copyStream(InputStream is, OutputStream os) throws IOException {
        return copyStream(is, os, new byte[4096]);
    }

    default long copyStream(InputStream is, OutputStream os, byte buffer[])
            throws IOException {
        int readLen;
        long totalCopied = 0;
        while ((readLen = is.read(buffer)) != -1) {
            os.write(buffer, 0, readLen);
            totalCopied += readLen;
        }
        return totalCopied;
    }

}
